package dev.shadowsoffire.apotheosis.potion.compat;

import net.minecraft.world.effect.MobEffectInstance;
import mezz.jei.api.ingredients.subtypes.UidContext;
import mezz.jei.api.gui.builder.IIngredientAcceptor;
import dev.shadowsoffire.apotheosis.ench.table.EnchantingRecipe;
import mezz.jei.api.gui.builder.IRecipeSlotBuilder;
import net.minecraft.world.item.crafting.Ingredient;
import java.util.Iterator;
import it.unimi.dsi.fastutil.ints.IntListIterator;
import dev.shadowsoffire.apotheosis.potion.PotionCharmItem;
import net.minecraft.world.item.alchemy.Potion;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.alchemy.Potions;
import java.util.stream.Collectors;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import net.minecraft.world.item.alchemy.PotionUtils;
import mezz.jei.api.ingredients.ITypedIngredient;
import java.util.function.Function;
import mezz.jei.api.recipe.IFocus;
import mezz.jei.api.ingredients.IIngredientType;
import mezz.jei.api.constants.VanillaTypes;
import net.minecraft.world.item.ItemStack;
import mezz.jei.api.recipe.IFocusGroup;
import mezz.jei.api.gui.builder.IRecipeLayoutBuilder;
import mezz.jei.api.recipe.category.extensions.vanilla.crafting.ICraftingCategoryExtension;
import net.minecraft.resources.ResourceLocation;
import mezz.jei.api.ingredients.subtypes.IIngredientSubtypeInterpreter;
import dev.shadowsoffire.apotheosis.Apoth;
import net.minecraft.world.item.Item;
import mezz.jei.api.registration.ISubtypeRegistration;
import dev.shadowsoffire.apotheosis.ench.compat.EnchantingCategory;
import dev.shadowsoffire.apotheosis.potion.PotionEnchantingRecipe;
import dev.shadowsoffire.apotheosis.potion.PotionCharmRecipe;
import mezz.jei.api.registration.IVanillaCategoryExtensionRegistration;
import dev.shadowsoffire.apotheosis.Apotheosis;
import mezz.jei.api.registration.IRecipeRegistration;
import mezz.jei.api.gui.ingredient.ICraftingGridHelper;
import mezz.jei.api.JeiPlugin;
import mezz.jei.api.IModPlugin;

@JeiPlugin
public class PotionJEIPlugin implements IModPlugin
{
    ICraftingGridHelper gridHelper;
    
    public void registerRecipes(final IRecipeRegistration reg) {
        if (!Apotheosis.enablePotion) {
            return;
        }
        this.gridHelper = reg.getJeiHelpers().getGuiHelper().createCraftingGridHelper();
    }
    
    public void registerVanillaCategoryExtensions(final IVanillaCategoryExtensionRegistration reg) {
        if (!Apotheosis.enablePotion) {
            return;
        }
        reg.getCraftingCategory().addCategoryExtension((Class)PotionCharmRecipe.class, x$0 -> new PotionCharmRecipeWrapper(x$0));
        EnchantingCategory.registerExtension((Class)PotionEnchantingRecipe.class, (EnchantingCategory.Extension)new PotionCharmEnchantingWrapper());
    }
    
    public void registerItemSubtypes(final ISubtypeRegistration reg) {
        if (!Apotheosis.enablePotion) {
            return;
        }
        reg.registerSubtypeInterpreter((Item)Apoth.Items.POTION_CHARM.get(), (IIngredientSubtypeInterpreter)new PotionCharmSubtypes());
    }
    
    public ResourceLocation getPluginUid() {
        return new ResourceLocation("apotheosis", "potion");
    }
    
    private class PotionCharmRecipeWrapper implements ICraftingCategoryExtension
    {
        private final PotionCharmRecipe recipe;
        
        PotionCharmRecipeWrapper(final PotionCharmRecipe recipe) {
            this.recipe = recipe;
        }
        
        public ResourceLocation getRegistryName() {
            return this.recipe.m_6423_();
        }
        
        public int getWidth() {
            return 3;
        }
        
        public int getHeight() {
            return 3;
        }
        
        public void setRecipe(final IRecipeLayoutBuilder builder, final ICraftingGridHelper craftingGridHelper, final IFocusGroup focuses) {
            final Potion potion = PotionUtils.m_43579_((ItemStack)focuses.getFocuses((IIngredientType)VanillaTypes.ITEM_STACK).findFirst().map((Function<? super Object, ?>)IFocus::getTypedValue).map((Function<? super Object, ? extends ItemStack>)ITypedIngredient::getIngredient).orElse(ItemStack.f_41583_));
            int i = 0;
            final List<List<ItemStack>> recipeInputs = this.recipe.m_7527_().stream().map(i -> Arrays.asList(i.m_43908_())).collect(Collectors.toCollection(ArrayList::new));
            if (potion != Potions.f_43598_) {
                final IntListIterator iterator = this.recipe.getPotionSlots().iterator();
                while (((Iterator)iterator).hasNext()) {
                    i = (int)((Iterator)iterator).next();
                    recipeInputs.set(i, Arrays.asList(PotionUtils.m_43549_(new ItemStack((ItemLike)Items.f_42589_), potion)));
                }
            }
            final ItemStack output = new ItemStack((ItemLike)Apoth.Items.POTION_CHARM.get());
            PotionUtils.m_43549_(output, potion);
            craftingGridHelper.createAndSetInputs(builder, (IIngredientType)VanillaTypes.ITEM_STACK, (List)recipeInputs, this.getWidth(), this.getHeight());
            if (potion != Potions.f_43598_) {
                craftingGridHelper.createAndSetOutputs(builder, (IIngredientType)VanillaTypes.ITEM_STACK, (List)Arrays.asList(output));
            }
            else {
                final List<ItemStack> potionStacks = new ArrayList<ItemStack>();
                for (final Potion p : ForgeRegistries.POTIONS) {
                    if (!PotionCharmItem.isValidPotion(p)) {
                        continue;
                    }
                    final ItemStack charm = new ItemStack((ItemLike)Apoth.Items.POTION_CHARM.get());
                    PotionUtils.m_43549_(charm, p);
                    potionStacks.add(charm);
                }
                craftingGridHelper.createAndSetOutputs(builder, (IIngredientType)VanillaTypes.ITEM_STACK, (List)potionStacks);
            }
        }
    }
    
    private class PotionCharmEnchantingWrapper implements EnchantingCategory.Extension<PotionEnchantingRecipe>
    {
        public void setRecipe(final IRecipeLayoutBuilder builder, final IRecipeSlotBuilder input, final IRecipeSlotBuilder output, final EnchantingRecipe recipe, final IFocusGroup focuses) {
            final ItemStack stack = focuses.getFocuses((IIngredientType)VanillaTypes.ITEM_STACK).findFirst().map((Function<? super Object, ?>)IFocus::getTypedValue).map((Function<? super Object, ? extends ItemStack>)ITypedIngredient::getIngredient).orElse(ItemStack.f_41583_);
            if (PotionCharmItem.hasEffect(stack)) {
                final ItemStack in = stack.m_41777_();
                in.m_41784_().m_128379_("Unbreakable", false);
                final ItemStack out = stack.m_41777_();
                out.m_41784_().m_128379_("Unbreakable", true);
                input.addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)in);
                output.addIngredient((IIngredientType)VanillaTypes.ITEM_STACK, (Object)out);
            }
            else {
                final List<ItemStack> potionStacks = new ArrayList<ItemStack>();
                final List<ItemStack> unbreakable = new ArrayList<ItemStack>();
                for (final Potion p : ForgeRegistries.POTIONS) {
                    if (!PotionCharmItem.isValidPotion(p)) {
                        continue;
                    }
                    final ItemStack charm = new ItemStack((ItemLike)Apoth.Items.POTION_CHARM.get());
                    PotionUtils.m_43549_(charm, p);
                    potionStacks.add(charm);
                    final ItemStack copy = charm.m_41777_();
                    copy.m_41784_().m_128379_("Unbreakable", true);
                    unbreakable.add(copy);
                }
                input.addIngredients((IIngredientType)VanillaTypes.ITEM_STACK, (List)potionStacks);
                output.addIngredients((IIngredientType)VanillaTypes.ITEM_STACK, (List)unbreakable);
            }
            builder.createFocusLink(new IIngredientAcceptor[] { (IIngredientAcceptor)input, (IIngredientAcceptor)output });
        }
    }
    
    private static class PotionCharmSubtypes implements IIngredientSubtypeInterpreter<ItemStack>
    {
        public String apply(final ItemStack stack, final UidContext context) {
            if (context == UidContext.Recipe) {
                return "";
            }
            if (!PotionCharmItem.hasEffect(stack)) {
                return "";
            }
            final MobEffectInstance contained = PotionCharmItem.getEffect(stack);
            return ForgeRegistries.MOB_EFFECTS.getKey((Object)contained.m_19544_()) + "@" + contained.m_19564_() + "@" + contained.m_19557_();
        }
    }
}
